/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2.1

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    Thanks : 4lj <4lj@flow.le.to> wrote patches to avoid source IP address
             spoofing problem.
             4lj <4lj@flow.le.to> implements "tmpfile hold" mode
             xcorp <xcorp@104.net> fixes bug in dump_packet()
             Flagmented TCP packet problem is reported by reverse.
             Packet length check problem is reported by reverse and hsj
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "depend.h"
#include "param.h"
#include "analyze.h"
#include "gen.h"
#include "error.h"
#include "streamio.h"
#include "gdd_printf.h"

CONN_LIST   *cl=NULL;       /* Connection table list */
CONN_LIST   *org_cl=NULL;   /* Base pointer of connection table list */

/*============================================================================
 Initalization of connection table list
 [INPUT]
  none
 [RETURN]
  -1    : Error
   0    : Success
  ============================================================================
*/
int init_connection_table(void)
{
    if ((cl=(CONN_LIST *)malloc(sizeof(CONN_LIST)))==NULL){
        errorlog(ERRLOG_FATAL,"Can not allocate memory\n");
        return(-1);
    }
    org_cl            = cl;
    cl->sourcePort    = 0;
    cl->destPort      = 0;
    strcpy(cl->sourceIP  ,"Start");
    strcpy(cl->destIP    ,"Start");
    cl->next_p        = (CONN_LIST *)malloc(sizeof(CONN_LIST));
    cl                = cl->next_p;
    cl->next_p        = NULL;
    return(0);
}

/*============================================================================
 Release connection table list
 [INPUT]
  none
 [RETURN]
  -1    : Error
   0    : Success
  ============================================================================
*/
int release_connection_table(void)
{
    CONN_LIST *ocl,*np;

    if (org_cl==NULL || cl==NULL) return(0);

    ocl=org_cl;
    np=ocl->next_p;
    for (;;){
        if (ocl==NULL) break;
        free(ocl);
        if (np==NULL) break;
        ocl=np;
        np=ocl->next_p;
    }
    return(0);
}

/*============================================================================
 Show content of connection table list (for debug purpose)
 [INPUT]
  none
 [RETURN]
  none
  ============================================================================
*/
void show_connection_table(void)
{
    CONN_LIST   *ncl;
    int         i;
	static char *logtypec[]={"BIN","TXT","XML","HTM"};
	char        lcs[16];

    ncl=org_cl;
    gdd_printf("\n>\n");
    gdd_printf("No.   SrcIP            SrcPort    DstIP            DstPort\n");
    for (i=0;;i++){
        if (ncl->next_p==NULL) break;
		if (i==0)
			strcpy(lcs,"");
		else if (ncl->logtype>=4)
			sprintf(lcs,"[%02X]",ncl->logtype);
		else
			sprintf(lcs,"[%s]",logtypec[ncl->logtype]);
        gdd_printf("%-5d %-16s %-5d  ->  %-16s %-5d %s\n",i,
                ncl->sourceIP, (int)ncl->sourcePort,
                ncl->destIP,   (int)ncl->destPort,lcs);
        ncl=ncl->next_p;    
    }
    gdd_printf("\n");
}

/*============================================================================
 Packet dump (for debug)
 [INPUT]
  pkt           : IP packet
 [RETURN]
  none
  ============================================================================
*/
void dump_packet(unsigned char *IP_packet,unsigned long length)
{
    struct ip       *ip_header;     /* IP header */
    struct tcphdr   *tcp_header;    /* TCP header */
    char            *tcp_data;      /* TCP data */
    struct in_addr  addr;           /* IP address */
    char            sourceIP[16];   /* Source IP address */
    char            destIP[16];     /* Destination IP address  */
    unsigned short  sourcePort;     /* Source Port */
    unsigned short  destPort;       /* Destination Port */
    unsigned long   len_data;       /* Length of data part */
    unsigned long   iph_len;        /* Length of IP header */
    unsigned long   tcph_len;       /* Length of TCP header */
    unsigned long   i,j;
    unsigned int    s,d;

    /* Get pointer and length of TCP/IP header */
    ip_header   = (struct ip *)IP_packet;
    iph_len     = (ip_header->ip_hl)*4;

    /* Get other parameter in TCP/IP header */
    memcpy(&addr,&(ip_header->ip_src),sizeof(struct in_addr));
    strcpy(sourceIP,(char *)inet_ntoa(addr));
    memcpy(&addr,&(ip_header->ip_dst),sizeof(struct in_addr));
    strcpy(destIP,(char *)inet_ntoa(addr));
    memcpy(&s,&ip_header->ip_src,sizeof(int));
    memcpy(&d,&ip_header->ip_dst,sizeof(int));
    
    /* Display IP packet info  */
    gdd_printf("-----------------------------------------------------------\n");
    gdd_printf("IP  :src(%s) dst(%s) Protocol(%d) header(%lubyte)\n",
        sourceIP,destIP,ip_header->ip_p,iph_len);
    gdd_printf("      hl=%x v=%x tos=%x len=%x id=%x off=%x\n",
        ip_header->ip_hl,ip_header->ip_v,ip_header->ip_tos,
        ntohs(ip_header->ip_len),ntohs(ip_header->ip_id),
        ntohs(ip_header->ip_off));
    gdd_printf("      ttl=%x p=%x sum=%x src=%x dst=%x\n",
        ip_header->ip_ttl,ip_header->ip_p,ntohs(ip_header->ip_sum),s,d);

    if (ip_header->ip_p==IPPROTO_TCP){
        tcp_header  = (struct tcphdr *)((char *)ip_header+iph_len);
        tcph_len    = ((unsigned long)(tcp_header->th_off))*4;
        tcp_data    = (char *)tcp_header+tcph_len;
        sourcePort  = ntohs(tcp_header->th_sport);
        destPort    = ntohs(tcp_header->th_dport);
        len_data    = (unsigned long)ntohs(ip_header->ip_len)
                      -iph_len-tcph_len;
        gdd_printf("TCP :sPort(%d) dPort(%d) hdr(%lubyte) data(%lubyte)\n",
            sourcePort,destPort,tcph_len,len_data);
        gdd_printf("      sp=%x dp=%x seq=%lx ack=%lx off=%x flg=%x\n",
            ntohs(tcp_header->th_sport),
            ntohs(tcp_header->th_dport),
            (unsigned long)ntohl(tcp_header->th_seq),
            (unsigned long)ntohl(tcp_header->th_ack),
            tcp_header->th_off&0xff,
            tcp_header->th_flags&0xff);
        gdd_printf("      win=%x sum=%x urp=%x\n",
            ntohs(tcp_header->th_win),
            ntohs(tcp_header->th_sum),
            ntohs(tcp_header->th_urp));
        gdd_printf("      l=%lu : il=%lu tl=%lu dl=%lu : totl=%lu\n",
            length,iph_len,tcph_len,len_data,
            iph_len+tcph_len+len_data);
    }

    for (i=0;i<length;i+=16){
        for (j=0;j<16;j++){
            if ((i+j)>=length) break;
            gdd_printf("%02X ",IP_packet[i+j]&0xff);
        }
        gdd_printf("\n");
    }
}

/*============================================================================
 Check port
 [INPUT]
  gddc      : Configuration of gdd
  sourcePort    : Source port number
  destPort  : Dest port number
 [RETURN]
 -1     : Error
 Otherwize  : Port list index
  ============================================================================
*/
int check_port(GDDCONFIG *gddc,
               unsigned short sourcePort,unsigned short destPort)
{
    int       i;

    for (i=0;i<=65536;i++){
        if (i==65536) return(-1);
        if (gddc->port[i].pst==ENDOFPORT) return(-1);
        if (( gddc->port[i].pst <= destPort
           && gddc->port[i].ped >= destPort)
          ||( gddc->port[i].pst <= sourcePort
           && gddc->port[i].ped >= sourcePort))
            break;
    }
    return(i);
}

/*============================================================================
 IP address matching
 [INPUT]
  mipaddr   : IPaddress which contains mask (ex. "192.168.0.0/24")
  ipaddr    : IPaddress (ex. "192.168.0.23")
 [RETURN]
  0     : Matched
 -1     : Not matched
  ============================================================================
*/
int ipaddr_match(char *mipaddr,char *ipaddr)
{
    char            *c;
    unsigned long   maskval,maskptn;
    unsigned long   vmipaddr,vipaddr;
    char            mipaddr_buf[MAXLEN_IPADDR];

    if (strlen(mipaddr)>=sizeof(mipaddr_buf)) return(-1);
    strcpy(mipaddr_buf,mipaddr);
    if ((c=strchr(mipaddr_buf,'/'))==NULL) return(-1);
    *c='\0';
    maskval=atoi(c+1);
    maskptn=((unsigned long)pow(2.0,(double)maskval)-1);
    vmipaddr=inet_addr(mipaddr_buf)&maskptn;
    vipaddr =inet_addr(ipaddr)&maskptn;

    return((vmipaddr==vipaddr)?0:-1);
}

/*============================================================================
 Check IPaddress
 [INPUT]
  gddc          : Configuration of gdd
  sourceIP      : Source IP address
  destIP        : Dest IP address
 [RETURN]
  0             : Success
 -1             : Error
  ============================================================================
*/
int check_IPaddr(GDDCONFIG *gddc,char *sourceIP,char *destIP)
{
    if (ipaddr_match(gddc->sourceIP,sourceIP)) return(-1);
    if (ipaddr_match(gddc->destIP  ,destIP))   return(-1);
    return(0);
}

/*============================================================================
 ACK/SEQ Check (FIN)
 [INPUT]
  gddc          : Configuration of gdd
  ncl           : Current connection
  tcp_header    : TCP header of current TCP packet
  len_data      : Length of current data
  sequence      : Expected sequence number
  sourceIP      : Source IP address
  sourcePort    : Source port
  destIP        : Dest IP address
  destPort      : Dest port
 [RETURN]
  0             : Success
 -1             : Error
  ============================================================================
*/
int SequenceCheck_FIN(
    GDDCONFIG *gddc,
    CONN_LIST *ncl,
    struct tcphdr *tcp_header, 
    unsigned long len_data,
    unsigned long sequence,
    char *sourceIP,unsigned short sourcePort,
    char *destIP,  unsigned short destPort)
{
    if (ncl->b_seq == tcp_header->th_seq 
        && ncl->b_ack == tcp_header->th_ack) {
        if (gddc->mode&MODE_DEBUG)
            gdd_printf("(--)\n");
    } else {
        if (sequence == tcp_header->th_ack){
            if (gddc->mode&MODE_DEBUG)
                gdd_printf("(OK)\n");
        } else {
            if ( ncl->b_ack == tcp_header->th_ack
              && ntohl(ncl->b_seq)+ntohl(ncl->b_dlen) ==
                 ntohl(tcp_header->th_seq)){
                if (gddc->mode&MODE_DEBUG)
                    gdd_printf("(**)\n");
            }else if (ntohl(sequence)
              ==ntohl(ncl->b_dlen)+ntohl(tcp_header->th_ack)){
                if (gddc->mode&MODE_DEBUG)
                    gdd_printf("(DUP?)\n");
            }else if (ntohl(sequence)
                    ==ntohl(ncl->b_dlen)
                     +ntohl(tcp_header->th_seq)){
                if (gddc->mode&MODE_DEBUG)
                    gdd_printf("(INTR?)\n");
            }else{
                errorlog(ERRLOG_WARNING,
                  "Illegal FIN/RST\n");
                errorlog(ERRLOG_WARNING,
                  "b_ack=%u th_ack=%u b_seq=%u th_seq=%u"\
                  "seq=%u b_dlen=%u\n",
                  ntohl(ncl->b_ack),ntohl(tcp_header->th_ack),
                  ntohl(ncl->b_seq),ntohl(tcp_header->th_seq),
                  ntohl(sequence),ntohl(ncl->b_dlen));
                errorlog(ERRLOG_WARNING,
                  "%s(%d)->%s(%d)\n",
                  sourceIP,sourcePort,destIP,destPort);
                return(-1);
            }
        }
    } 
    ncl->b_seq  = tcp_header->th_seq;
    ncl->b_ack  = tcp_header->th_ack;
    ncl->b_dlen = htonl(len_data);
    return(0);
}

/*============================================================================
 ACK/SEQ Check (DATA)
 [INPUT]
  gddc          : Configuration of gdd
  ncl           : Current connection
  tcp_header    : TCP header of current TCP packet
  len_data      : Length of current data
  sequence      : Expected sequence number
  sourceIP      : Source IP address
  sourcePort    : Source port
  destIP        : Dest IP address
  destPort      : Dest port
 [RETURN]
  0             : Success
 -1             : Error
  ============================================================================
*/
int SequenceCheck_DATA(
    GDDCONFIG *gddc,
    CONN_LIST *ncl,
    struct tcphdr *tcp_header, 
    unsigned long len_data,
    unsigned long sequence,
    char *sourceIP,unsigned short sourcePort,
    char *destIP,  unsigned short destPort)
{

    if (ncl->b_ack == tcp_header->th_ack
        && (ncl->b_seq != tcp_header->th_seq
        || ntohl(ncl->b_seq)+ntohl(ncl->b_dlen)
         ==ntohl(tcp_header->th_seq))){
        if (gddc->mode&MODE_DEBUG)
            gdd_printf("(--)\n");
    } else {        
        if (sequence == tcp_header->th_ack){
            if (gddc->mode&MODE_DEBUG)
                gdd_printf("(OK)\n");
        } else {
            if (ntohl(sequence)
              ==ntohl(ncl->b_dlen)+ntohl(tcp_header->th_ack)){
                if (gddc->mode&MODE_DEBUG)
                    gdd_printf("(DUP?)\n");
            }else if (ntohl(sequence)
              ==ntohl(ncl->b_dlen)+ntohl(tcp_header->th_seq)){
                if (gddc->mode&MODE_DEBUG)
                    gdd_printf("(INTR?)\n");
            } else {
                errorlog(ERRLOG_WARNING,
                  "Illegal sequence number\n");
                errorlog(ERRLOG_WARNING,
                  "b_ack=%u th_ack=%u b_seq=%u th_seq=%u"\
                  "seq=%u b_dlen=%u\n",
                  ntohl(ncl->b_ack),ntohl(tcp_header->th_ack),
                  ntohl(ncl->b_seq),ntohl(tcp_header->th_seq),
                  ntohl(sequence),ntohl(ncl->b_dlen));
                errorlog(ERRLOG_WARNING,
                  "%s(%d)->%s(%d)\n",
                  sourceIP,sourcePort,destIP,destPort);
                if (gddc->mode&MODE_SECURE)
                    return(-1);
            }
        }
    }
    ncl->b_seq  = tcp_header->th_seq;
    ncl->b_ack  = tcp_header->th_ack;
    ncl->b_dlen = htonl(len_data);
    return(0);
}

/*============================================================================
 Force terminate logging
 [INPUT]
  gddc      : Configuration of gdd
 [RETURN]
  0     : Success
 -1     : Error
  ============================================================================
*/
int fourceterm_connection(GDDCONFIG *gddc)
{
	CONN_LIST       *ncl;

    for (;;){
		ncl=org_cl->next_p;
		if (ncl==NULL || ncl->next_p==NULL)
            break;		
		streamlogging_close(gddc,ncl->start_time,ncl->logtype,
						  ncl->sourceIP,ncl->sourcePort,
						  ncl->destIP,ncl->destPort);
		org_cl->next_p=ncl->next_p;
		free((void *)ncl);
    }
	gdd_printf("\nConnections were cleaned up.\n");
    if (gddc->mode&MODE_DEBUG)
        show_connection_table();
	return(0);
}
/*============================================================================
 enum connections
 [INPUT]
  gddc      : Configuration of gdd
 [RETURN]
  number of connections
  ============================================================================
*/
int enum_connection(GDDCONFIG *gddc)
{
	CONN_LIST       *ncl;
	int             c;

	ncl=org_cl->next_p;
    for (c=0;;c++){
		if (ncl==NULL || ncl->next_p==NULL)
            break;
		ncl=ncl->next_p;
    }
	return(c);
}

/*============================================================================
 Analyze sniffed packet and append to logfile
 [INPUT]
  gddc      : Configuration of gdd
  packet    : Sniffed packet
  length    : Length of sniffed packet
 [RETURN]
  0     : Success
 -1     : Error
  ============================================================================
*/
int packet_analysis(GDDCONFIG *gddc,unsigned char *packet,unsigned long length)
{
    struct ip       *ip_header;     /* IP header  */
    struct tcphdr   *tcp_header;    /* TCP header */
    char            *tcp_data;      /* TCP data */
    struct in_addr  addr;           /* IP address */
    char            sourceIP[16];   /* Source IP address */
    char            destIP[16];     /* Destination IP address  */
    unsigned short  sourcePort;     /* Source Port */
    unsigned short  destPort;       /* Destination Port */
    unsigned long   len_data;       /* Length of data part */
    unsigned long   iph_len;        /* Length of IP header */
    unsigned long   tcph_len;       /* Length of TCP header */
    unsigned long   sequence;       /* Expected sequence */
    int             portindex;      /* Indexnumber of port list */
    int             direction;      /* Packet direction */
    unsigned char   logtype;        /* Log type */
    CONN_LIST       *bcl,*ncl;      /* Connection table list */
    CONN_LIST       *t;             /* Temporary connection list */
    static char     datestr[512];   /* Buffer to store datetime */
    time_t          timeval;
    struct tm       *timep=NULL;
    char            *timesp=NULL;
    char            *c;

    /* Get pointer of IP header and check length of IP */
    if (length-SIZE_OF_ETHHDR < MINSIZE_IP+MINSIZE_TCP) return(0);
    ip_header   = (struct ip *)(packet+SIZE_OF_ETHHDR);
    if (ip_header->ip_p!=IPPROTO_TCP
     || ip_header->ip_v!=4) return(0);
    iph_len     = ((unsigned long)(ip_header->ip_hl))*4;
    if (iph_len<MINSIZE_IP) return(0);
    if ((unsigned long)ntohs(ip_header->ip_len) < MINSIZE_IP+MINSIZE_TCP)
        return(0);
    if ((unsigned long)ntohs(ip_header->ip_len) > length-SIZE_OF_ETHHDR){
        return(0);
    }

    /* Get pointer of TCP header and check length of TCP */
    tcp_header  = (struct tcphdr *)((char *)ip_header+iph_len);
    tcph_len    = ((unsigned long)(tcp_header->th_off))*4;
    tcp_data    = (char *)tcp_header+tcph_len;
    if (tcph_len<MINSIZE_TCP) return(0);

    /* Get other parameter in TCP/IP header */
    if ((long)ntohs(ip_header->ip_len)-(long)iph_len-(long)tcph_len<0)
        return(0);
    len_data    = (unsigned long)ntohs(ip_header->ip_len)
                      -iph_len-tcph_len;
    sourcePort  = ntohs(tcp_header->th_sport);
    destPort    = ntohs(tcp_header->th_dport);
    memcpy(&addr,&(ip_header->ip_src),sizeof(struct in_addr));
    strcpy(sourceIP,(char *)inet_ntoa(addr));
    memcpy(&addr,&(ip_header->ip_dst),sizeof(struct in_addr));
    strcpy(destIP,(char *)inet_ntoa(addr));
    if (!strcmp(sourceIP,destIP)) return(0);

    /* Port check */
    if ((portindex=check_port(gddc,sourcePort,destPort))==-1) return(0);
    logtype=gddc->port[portindex].logtype;

    /* dump_packet */
    if (gddc->mode&MODE_DUMP)
        dump_packet((unsigned char *)ip_header,length-SIZE_OF_ETHHDR);

    /* First step to logging (SYN) */
    if (tcp_header->th_flags & TH_SYN && tcp_header->th_flags & TH_ACK){

        /* IP address filtering */
        if (check_IPaddr(gddc,destIP,sourceIP)) return(0);
        
        /* Is this fragmentation packet? */
        if (ntohs(ip_header->ip_off)!=0x4000
         && ntohs(ip_header->ip_off)!=0x0000){
            errorlog(ERRLOG_WARNING,"Fragmentation detected\n");
            streamlogging_interrupt(gddc,"Fragmentation (SYN+ACK) ",
                tcp_data,
                len_data,1,
                sourceIP,sourcePort,
                destIP,destPort);
            return(0);
        }

        /* Is this connection alredy in connection table? */
        ncl=org_cl->next_p;
        for (;;){
            if (ncl->next_p==NULL) break;
            if (ncl->sourcePort == destPort
             && ncl->destPort   == sourcePort
             && !strcmp(ncl->sourceIP,destIP)
             && !strcmp(ncl->destIP,sourceIP)){
                errorlog(ERRLOG_WARNING,
                    "SYN sent from same IP and port.\n");
                return(0);
            }
            if ((ncl=ncl->next_p)==NULL) break;
        }

        /* store date&time to datestr */
        timeval=time(NULL);
        if (timeval!=(time_t)-1){
            timep=localtime(&timeval);
            if (timep!=NULL)
                timesp=asctime(timep);
        }
        if (timesp==NULL)
            strcpy(datestr,"time() error ");
        else{
            strncpy(datestr,timesp,sizeof(datestr)-1);
            datestr[sizeof(datestr)-1]='\0';
            if ((c=strchr(datestr,'\r'))!=NULL
             || (c=strchr(datestr,'\n'))!=NULL) *c='\0';
        }
        
        /* Set information to connection table list */
        cl->next_p=(CONN_LIST *)malloc(sizeof(CONN_LIST));
        if (cl->next_p==NULL){
            errorlog(ERRLOG_FATAL,"Can not allocate memory\n");
            return(-1);
        }
        
        strncpy(cl->start_time,datestr,sizeof(cl->start_time));
        strcpy(cl->sourceIP,destIP);
        strcpy(cl->destIP,sourceIP);
        cl->sourcePort  = destPort;
        cl->destPort    = sourcePort;
        cl->SYNtick     = (unsigned long)time(NULL);
        cl->b_seq       = tcp_header->th_seq;
        cl->b_ack       = tcp_header->th_ack;
        cl->b_dlen      = htonl((len_data? len_data:1));
        cl->state       = STATE_SYN;
		cl->logtype     = logtype;
        cl              = cl->next_p;
        cl->next_p      = NULL;
        if (gddc->mode&MODE_DEBUG)
            show_connection_table();

        /* Write connection information to logfile */
        streamlogging_open(gddc,datestr,logtype,
                           destIP,destPort,
                           sourceIP,sourcePort);
        if ((gddc->mode&MODE_DEBUG)&&(gddc->mode&MODE_SECURE))
            gdd_printf("\n-> S:%lu   A:%lu   D:%lu\n",
                (unsigned long)ntohl(tcp_header->th_seq),
                (unsigned long)ntohl(tcp_header->th_ack),
                (unsigned long)len_data);
        return(0);
    }

    /* Get this connection from connection table list */
    ncl=org_cl->next_p;  /* This connection */
    bcl=org_cl;          /* Connection previous this connection */
    direction=-1;        /* 0=Positive direction, 1=Negative direction */
    for (;;){
        if (ncl->next_p==NULL){
            ncl=NULL;
            break;
        }
        /* SYN timeout check */
        if (ncl->state==STATE_SYN){
            if ((unsigned long)time(NULL)-ncl->SYNtick 
                >= TH_SYNTIMEOUT){
                errorlog(ERRLOG_WARNING,
                    "SYN timeout %s(%lu)->%s(%lu)\n",
                    ncl->sourceIP,ncl->sourcePort,
                    ncl->destIP,ncl->destPort);
                streamlogging_close(gddc,ncl->start_time,logtype,
                    ncl->sourceIP,ncl->sourcePort,
                    ncl->destIP,ncl->destPort);
                bcl->next_p=ncl->next_p;
                t=ncl->next_p;
                free((void *)ncl);
                if (gddc->mode&MODE_DEBUG)
                    show_connection_table();
                if ((ncl=t)==NULL){
                    ncl=NULL;
                    break;
                }
                continue;
            }
        }

        /* Port/IPaddress matching */
        if (ncl->sourcePort == sourcePort 
         && ncl->destPort   == destPort
         && !strcmp(ncl->sourceIP,sourceIP)
         && !strcmp(ncl->destIP,destIP)){
            direction=0;
            break;
        }
        if (ncl->sourcePort == destPort
         && ncl->destPort   == sourcePort
         && !strcmp(ncl->sourceIP,destIP)
         && !strcmp(ncl->destIP,sourceIP)){
            direction=1;
            break;
        }
        bcl=ncl;
        if ((ncl=ncl->next_p)==NULL){
            ncl=NULL;
            break;
        }
    }    
    
    /* IP address filtering */
    if (ncl != NULL || direction >= 0) {
        if (direction==0){
            if (check_IPaddr(gddc,sourceIP,destIP)) return(0);
        }else{
            if (check_IPaddr(gddc,destIP,sourceIP)) return(0);
        }
    }

    /* Is this fragmentation packet? */
    if (ncl!=NULL && direction >=0
     && ntohs(ip_header->ip_off)!=0x4000
     && ntohs(ip_header->ip_off)!=0x0000){
        errorlog(ERRLOG_WARNING,"Fragmentation detected\n");
        streamlogging_interrupt(gddc,"Fragmentation ",
            tcp_data,
            len_data,direction,
            sourceIP,sourcePort,
            destIP,destPort);
        streamlogging_close(gddc,ncl->start_time,logtype,
                          ncl->sourceIP,ncl->sourcePort,
                          ncl->destIP,ncl->destPort);

        bcl->next_p=ncl->next_p;
        free((void *)ncl);
        if (gddc->mode&MODE_DEBUG)
            show_connection_table();
        return(0);
    }

    /* Calculate expected sequence number */
    sequence = 0;
    if (ncl != NULL || direction >= 0) {
        if (ncl->state == STATE_SYN) {
            if (len_data == 0) len_data = 1;
        }
        sequence = htonl(ntohl(ncl->b_seq) + ntohl(ncl->b_dlen));
        if ((gddc->mode&MODE_DEBUG)&&(gddc->mode&MODE_SECURE)){
            switch (direction)
                {
                case 0:
                    gdd_printf("\n-> ");
                    break;
                case 1:
                    gdd_printf("\n<- ");
                    break;
                default:
                    gdd_printf("\n?? ");
                    break;
                }   
            gdd_printf("S:%lu   A:%lu   D:%lu   C:%lu   ",
                (unsigned long)ntohl(tcp_header->th_seq),
                (unsigned long)ntohl(tcp_header->th_ack),
                (unsigned long)len_data,
                (unsigned long)ntohl(sequence));
        }
    }
    
    /* Processes for connection close */
    if (tcp_header->th_flags & TH_FIN || tcp_header->th_flags & TH_RST){

        if (ncl==NULL) return(0);   
        if ((gddc->mode&MODE_SECURE)){
            if (SequenceCheck_FIN(gddc,ncl,tcp_header,len_data,
                                  sequence,
                                  sourceIP,sourcePort,
                                  destIP,destPort))
            return(0);
        }

        /* Append TEMP log to real logfile */
        streamlogging_close(gddc,ncl->start_time,logtype,
                          ncl->sourceIP,ncl->sourcePort,
                          ncl->destIP,ncl->destPort);

        bcl->next_p=ncl->next_p;
        free((void *)ncl);
        if (gddc->mode&MODE_DEBUG)
            show_connection_table();
        if ((gddc->mode&MODE_DEBUG)&&(gddc->mode&MODE_SECURE))
            gdd_printf("\n--- finish ---\n");
        return(0);
    }

    /* Process for TCP data */
    if (!(tcp_header->th_flags & TH_ACK)) return(0);
    if (ncl==NULL || direction==-1){
        if (len_data>0){
            streamlogging_interrupt(gddc,"Unexpected DATA ",
                tcp_data,len_data,direction,
                sourceIP,sourcePort,
                destIP,destPort);
        }
        return(0);
    }
    if (ncl->state==STATE_SYN){
        if (gddc->mode&MODE_DEBUG)
            gdd_printf("%s(%u) -> %s(%u) is established.\n",
                ncl->sourceIP,ncl->sourcePort,
                ncl->destIP,ncl->destPort);
        ncl->state=STATE_ESTABLISHED;
    }
    
    if ((gddc->mode&MODE_SECURE)){
        if (SequenceCheck_DATA(gddc,ncl,tcp_header,len_data,sequence,
                              sourceIP,sourcePort,destIP,destPort))
        return(0);
    }

    /* Append to TEMP logfile */
    streamlogging_write(gddc,ncl->start_time,tcp_data,len_data,direction,logtype,
        ncl->sourceIP,ncl->sourcePort,ncl->destIP,ncl->destPort);
    return(0);
}

