/*****************************************************************************
    Ethernet packet sniffer - GreedyDog2

    Developed by Shadow Penguin Security Resarch Group
     http://www.shadowpenguin.org
     UNYUN <unyun@shadowpenguin.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

 *****************************************************************************/

#include <stdio.h>
#include "depend.h"
#include "param.h"
#include "gddconfig.h"
#include "error.h"
#include "gui_win32_console.h"
#include "gui_win32_msghdlr_main.h"
#include "gui_win32_settingwndproc.h"
#include "gdd_printf.h"
#include "resource.h"
#include <commctrl.h>
#include <shlobj.h>

/*============================================================================
 Callback procedure of setting Dialog Box
 [INPUT]
  hWnd      : Window handle of main window
  msg       : Message
  wParam    : Parameter 1
  lParam    : Parameter 2
 [RETURN]
  none
  ============================================================================
*/
int	CALLBACK SettingDlgProc(HWND hDlg,UINT msg,WPARAM wParam,LPARAM lParam)
{
	void set_defaultvalue(GDDCONFIG *gddc);
	char *SaveAsDialogBox(HWND hDlg);
	char *OpenDirectoryDialogBox(HWND hDlg);
	int  GetParamsFromStgDlg(HWND hDlg,GDDCONFIG *gddc);

	static GDDCONFIG    *gddc;
	GDDGUI              gddgui;
	char                *p;
	int                 r;

	switch(msg){
		case WM_INITDIALOG:
			gddc=(GDDCONFIG *)lParam;
			SetDlgItemText(hDlg,IDC_LOGFILE,gddc->logfile);
			SetDlgItemText(hDlg,IDC_ERRLOGFILE,gddc->errlog);
			SetDlgItemText(hDlg,IDC_TMPLOGDIR,gddc->templogdir);
			SetDlgItemText(hDlg,IDC_ACTION,gddc->action);
			CheckDlgButton(hDlg,IDC_DEBUG,gddc->mode&MODE_DEBUG);
			break;
		case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam, lParam)){
				case IDCANCEL:
					EndDialog(hDlg,0);
					return TRUE;
				case IDOK:
					GetParamsFromStgDlg(hDlg,gddc);
					EndDialog(hDlg,1);
					return TRUE;
				case IDC_REF_LOGFILE:
					if ((p=SaveAsDialogBox(hDlg)))
						SetDlgItemText(hDlg,IDC_LOGFILE,p);
					break;
				case IDC_REF_ERRLOGFILE:
					if ((p=SaveAsDialogBox(hDlg)))
						SetDlgItemText(hDlg,IDC_ERRLOGFILE,p);
					break;
					if ((p=OpenDirectoryDialogBox(hDlg)))
						SetDlgItemText(hDlg,IDC_TMPLOGDIR,p);
				case IDC_HELPGDD:
					ShellExecute(hDlg,"open",FILE_README,NULL,NULL,SW_SHOWNORMAL);
					break;
				case IDC_UPDATE:
					ShellExecute(hDlg,"open",URL_GDD,NULL,NULL,SW_SHOWNORMAL);
					break;
				case IDC_DEFAULT:
					r=MessageBox(hDlg,"Reset all parameters now ?","WARNING",MB_YESNO);
					if (r==IDYES){
						gddgui=gddc->egui;
						set_defaultvalue(gddc);
						gddc->egui=gddgui;
						Win32MsgSubfunc_SetParamsToMainDlg(gddc);
						SendMessage(hDlg,WM_INITDIALOG,0,(LPARAM)gddc);
					}
					break;
				case IDC_SAVE:
					GetParamsFromStgDlg(hDlg,gddc);
					if (Win32MsgSubfunc_GetParamsFromMainDlg(gddc,0)!=0){
						MessageBox(hDlg,"Can not save due to parameter error(s)",
							"ERROR",MB_OK);
						break;
					}
					Win32MsgSubfunc_SaveCFG(gddc);
					break;
			}
			break;
	}
	return FALSE;
}

/*============================================================================
 Display "Save As" dialog box
 [INPUT]
  hWnd      : Window handle of main window
 [RETURN]
  NULL = Cancel
  otherwise, path of specified file
  ============================================================================
*/
char *SaveAsDialogBox(HWND hDlg)
{
	OPENFILENAME	ofn;
	HINSTANCE		hInst=GetModuleHandle(0);
	char			szFilterPattern[256];
	static char		szOutputFilename[MAX_PATH];

	memset(&ofn,0,sizeof(OPENFILENAME));
	memset(szFilterPattern,0,sizeof(szFilterPattern));
	strcpy(szFilterPattern,"*.*"); 

	ofn.lStructSize     = sizeof(OPENFILENAME);
	ofn.hwndOwner       = hDlg;
	ofn.hInstance       = hInst;
	ofn.lpstrFilter     = szFilterPattern;
	ofn.lpstrFile       = szOutputFilename;
	ofn.nMaxFile        = MAX_PATH;
	ofn.Flags           = OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST|OFN_EXPLORER;
	if (GetSaveFileName(&ofn)==FALSE) return NULL;
	if (strlen(szOutputFilename)==0) return NULL;
	return(szOutputFilename);
}

/*============================================================================
 Get parameters from setting dialog box
 [INPUT]
  hDlg      : Window handle of main window
 [RETURN]
  NULL = Cancel
  otherwise, path of specified file
  ============================================================================
*/
int GetParamsFromStgDlg(HWND hDlg,GDDCONFIG *gddc)
{
	GetDlgItemText(hDlg,IDC_LOGFILE,gddc->logfile,MAX_PATH);
	GetDlgItemText(hDlg,IDC_ERRLOGFILE,gddc->errlog,MAX_PATH);
	GetDlgItemText(hDlg,IDC_TMPLOGDIR,gddc->templogdir,MAX_PATH);
	GetDlgItemText(hDlg,IDC_ACTION,gddc->action,MAXLEN_ACTION);
	if (IsDlgButtonChecked(hDlg,IDC_DEBUG)) gddc->mode|=MODE_DEBUG;
	else gddc->mode&=!MODE_DEBUG;
	return(0);
}
